/*
(C) GenFail, 2019
genfail@hotmail.com

GNFLChrono V1.4 (для Atmega8, Atmega168 и Atmega328).
Теперь используется таймер 1 без предделителя. 

ВАЖНО!!!
Не забываем выставить длину базы.
Фототранзистора два, не забываем переключаться между ними в сервисном режиме.
На платах с чипом LGT8F328P (в основном клоны от Wavgat) работать не будет.
Если питается от USB - нормально работать не будет! Необходимо питание через vin.
Оптимальное напряжение - 1.8в (лучше ставить внешний компаратор LM393).

Используется библиотека GyverTM1637, спасибо AlexGyver.
*/

//Для совместимости с Atmega8
#if (defined (TIMSK) && ! defined (TIMSK1))
#define TIMSK1  TIMSK
#endif

#include <GyverTM1637.h>
#include <EEPROM.h>

//пины дисплея TM1637
#define CLK 5
#define DIO 4

//пин кнопки и питание кнопки
#define BUTTON_PIN 8
#define BUTTON_POWER 9

//аналоговые входы для датчиков
#define ANALOG_0 4
#define ANALOG_1 2

#define VOLTAGE 5.0 //напряжение питания.

//это не трогать
#define STAND_BY 0
#define CALCULATING 1
#define TIMING 2

GyverTM1637 disp(CLK, DIO);
byte flag_timer = 0;
boolean flag_button = 0;
byte counter = 0;
float dist=0.0700; //расстояние между датчиками
byte chron_state;
unsigned long timer;
int velocity;
int velocity1;
byte state = 1;
byte button = 1;
long time_press;
byte overflowCounter = 0;


void serviceMode() //сервисный режим, показываем напряжение на фототранзисторах
{
  byte service[] = {_S, _E, _r, _U, _1, _C, _E};
  disp.runningString(service, sizeof(service), 200);
  pinMode(2, INPUT);
  pinMode(3, INPUT);
  byte sw = ANALOG_0;
  disp.point(1); //включим точки
  float voltage;
  while(1)
  {
    voltage = (analogRead(sw)*VOLTAGE/1023.0*100.0); //измеряем напряжение
    disp.displayInt(round(voltage));
    delay(100);
    //если нажата кнопка, переключаем вход
    if (digitalRead(BUTTON_PIN) == 1)
    {
      delay(100);
      state=1;
      button=1;      
      time_press=millis();  
      while (millis()-time_press<5000)
      {        
        if (digitalRead(BUTTON_PIN)==0) 
        {
          button=0;
          break;
        }
      }
      if (button==0) 
      {   
          
          if (sw == ANALOG_0) 
            {
              sw=ANALOG_1;
            }
          else
            {
              sw=ANALOG_0;
            }
          disp.clear();
          if (sw == ANALOG_0)
            {
              disp.display(1, 1);
            }
          if (sw == ANALOG_1)
            {
              disp.display(1, 2);
            }
          delay(1000);
      }
      if (button==1) 
      {
          erase();
      } 
    }
    if (digitalRead(BUTTON_PIN)==0 && state==1) 
      {     
          state=0;  
      } 
  }
}

//стираем EEPROM
void erase()
{
    disp.point(0);
    byte erase[] = {_E, _r, _A, _S, _E};
    disp.runningString(erase, sizeof(erase), 200);
    for (int i = 0; i < 512; i++)
      {
        EEPROM.update(i, 0);
      }
    EEPROM.put(0, 0);
    EEPROM.write(200, 123);
    byte done[] = {_d, _O, _n, _E};
    disp.runningString(done, sizeof(done), 200);
    disp.clear();
    disp.point(1);
}

// Инит таймера 1
void initSensors() //предделитель 0;
{
  cli();
  attachInterrupt(1, sensorOne, RISING);
  attachInterrupt(0, sensorTwo, RISING);
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  TIMSK1 = 0;
  TIMSK1 |= (1<<TOIE1); //прерывание по переполнению таймера
  sei();
}

void sensorOne()
{
        if (chron_state == STAND_BY) //а если датчик сработает от свинцовой пыли после пули, то таймер снова не запустится.
        {
          TCCR1B = (1<<CS10); // Запустим таймер
          chron_state = TIMING; 
        }       
}

void sensorTwo()
{
        if (chron_state == TIMING)
        {
          TCCR1B = 0; // Остановим таймер
          chron_state = CALCULATING;
        }
        else if (chron_state == STAND_BY)
        {
          displayError(2);
        }
}

ISR(TIMER1_OVF_vect)
{
  if (chron_state == TIMING)
  {
    if (overflowCounter < 15)
    {
      overflowCounter++;
    }
    else
    {
      TCCR1B = 0;
      TCNT1 = 0;
      displayError(1);
      overflowCounter = 0;
      chron_state=STAND_BY;
    }
    }
}

void displayError(int error)
{
  byte err1[] = {_E, _1, _empty, _empty};
  byte err2[] = {_E, _2, _empty, _empty};
  byte err3[] = {_E, _3, _empty, _empty};
  byte err4[] = {_F, _u, _l, _l};
  byte err5[] = {_under, _under, _under, _under};
  switch ( error ) {
    case 1:
    disp.clear();
    disp.displayByte(err1);
    EEPROM.get(0, counter);
    if (counter > 2)
      {
        counter += 2;
      }
    break;
    
    case 2:
    disp.clear();
    disp.displayByte(err2);
    EEPROM.get(0, counter);
    if (counter > 2)
      {
        counter += 2;
      }
    break;
    
    case 3:
    disp.clear();
    disp.displayByte(err3);
    break;
    
    case 4:
    disp.clear();
    disp.displayByte(err4);
    break;

    case 5:
    disp.clear();
    disp.displayByte(err5);
    break;
  }
  
   return 0;
}

void buttonPress() //а вынесем-ка нажатие кнопки в отдельную функцию
{
state=1;
button=1;      
time_press=millis();  
while (millis()-time_press<5000)
{       
  if (digitalRead(BUTTON_PIN)==0) 
  {
    button=0;
    break;
  }
  }
  if (button==0) 
  {   
  //при коротком нажатии
    if (counter == 0)
    {
      displayBlink(1);
      displayError(5);
    }
    else
    {
      if (counter != 2)
        {
          counter -= 2;
        }
       else if (counter == 2)
       {
         EEPROM.get(0, counter);
       }
      EEPROM.get(counter, velocity1);
      disp.point(1); 
      disp.clear();
      //disp.displayInt(counter/2);
      disp.display(0, (counter/2)/10); //костыль для красивого отображения
      disp.display(1, (counter/2)%10);
      displayBlink(1);
      //delay(500);
      disp.point(0);
      disp.clear();
      disp.displayInt(velocity1); 
    }
  }
  if (button==1) 
  {
  //при длинном
  erase();
  disp.point(0);
  EEPROM.get(0, counter);
  displayError(5);
  } 
  if (digitalRead(BUTTON_PIN)==0 && state==1) 
  {     
  state=0;  
  } 
}

void calculateAndShowSpeed()
{
   velocity = round((F_CPU / 1000L) * (dist*1000.0) / (TCNT1 + (overflowCounter*65535)));
   disp.clear();
   disp.brightness(7);
   if (velocity > 0)
    {
    EEPROM.get(0, counter);
    if (counter < 100)
      {
        if (counter == 0)
        {
        counter = 2;
        }
        else
        {
          counter += 2;
        }
        EEPROM.put(counter, velocity);
        EEPROM.put(0, counter);
        disp.clear();
        disp.point(1);
        disp.display(0, (counter/2)/10); //костыль для красивого отображения
        disp.display(1, (counter/2)%10);
        //disp.displayInt(counter/2);
        delay(1000);
        disp.point(0);
        disp.clear();
        disp.displayInt(velocity);
      }
    else
      {
        EEPROM.get(0, counter);
        counter+=2;
        displayError(4);
        delay(1000);
        disp.clear();
        disp.displayInt(velocity);
      }
    }
   TCNT1 = 0; //обнулим таймер
   overflowCounter = 0; //cчетчик переполнений обнулим
   chron_state = STAND_BY;
}

void displayBlink(int repeat)
{
  while (repeat>0)
  {
    for (int i=7; i>=0; i--)
    {
      disp.brightness(i);
      delay(50);
    }
    for (int i=0; i<=7; i++)
    {
      disp.brightness(i);
      delay(15);
    }
    repeat--;
  }
  
  return 0;
}

void welcome()
{
  byte welcome_banner[] = {_G, _n, _F, _L};
  disp.displayByte(welcome_banner);
  displayBlink(1);
}

void setup() {
  disp.clear();
  disp.brightness(7); //яркость дисплея
  pinMode(14, INPUT);
  digitalWrite(14, HIGH);
  pinMode(BUTTON_POWER, OUTPUT); //питание кнопки
  digitalWrite(BUTTON_POWER, HIGH); //включаем питание кнопки
  pinMode(BUTTON_PIN, INPUT); //пин кнопки
  
  if (digitalRead(BUTTON_PIN) == 1) //если в момент включения нажата кнопка, то включаем сервисный режим
    {
      serviceMode();
    }
  
  if ((digitalRead(2) == 1) || (digitalRead(3) == 1)) //если перекрыты или неисправны датчики, предупредим
    {
      displayError(3);
      while(1)
      {
      displayBlink(1); 
      if (digitalRead(BUTTON_PIN) == 1)
        {
          disp.clear();
          break;
        }
      }
    }
  initSensors();
  welcome();
  EEPROM.get(0, counter);
  if (counter != 0)
  {
    disp.point(1); 
    disp.clear();
    //disp.displayInt(counter/2);
    disp.display(0, (counter/2)/10); //костыль для красивого отображения
    disp.display(1, (counter/2)%10);
    displayBlink(1);
    //delay(500);
    disp.point(0);
    disp.clear();
    EEPROM.get(counter, velocity1);
    disp.displayInt(velocity1); //отобразим последнее измерение
  }
  else
  {
    displayError(5);
  }
  disp.point(0);
  delay(1000); //чтобы не сработало второй раз на облако газа от СО2
  chron_state = STAND_BY;
}

void loop() 
{
if ((digitalRead(BUTTON_PIN) == 1) && (chron_state == STAND_BY))
    {
      buttonPress();
    }
if (chron_state == CALCULATING)
{
  calculateAndShowSpeed();
}
}
